/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             Log management
*  SCOPE :              Log management
*  FILE :               log.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __log_HXX
#define __log_HXX

/* IMPORTS *******************************************************************/

#ifdef __hpux__
extern "C" {
#endif
#include <sys/mman.h>
#ifdef __hpux__
}
#endif
  
#include <unistd.h>

#ifndef __class_HXX
#include "class.hxx"
#endif

#ifndef __yoofunc_HXX
#include "yoofunc.hxx"
#endif

#ifndef __oid_HXX
#include "oid.hxx"
#endif

#include "alphadecl.h"


/*****************************************************************************/

class Log {

  protected :

  char* _file ;
  int _fd ;
  int _size ;
  int* _inuse ;
  PID* _addresses ;
  static const int MapSize ;
  
  public :

  Log(char* f) ;                   // Build a new log a read an old one
  virtual ~Log() ;                 // unlink log file
  virtual PID read(Page*, int pagenum) ;   // Read a page from log
  virtual int write(Page*, PID) ;          // Write a page into log
  virtual void close() ;           // Set log as inactive log
  void sync() ;                    // Synchronise log
  int  size() ;                    // Return current size of the log
  int isactive() ;                 // Return TRUE if log is active
} ;


inline Log::~Log() {
  delete _file ;
  SYSCALLI(munmap((caddr_t)_inuse, PAGE_SIZE)) ;
  SYSCALLI(munmap((caddr_t)_addresses, MapSize)) ;
  SYSCALLI(unlink(_file)) ;
}

inline void Log::sync() {
  SYSCALLI(fsync(this->_fd)) ;
}

inline int Log::size() {
  return this->_size ;
}

inline void Log::close() {
  *this->_inuse = FALSE ;
  this->sync() ;
  ::close(this->_fd) ;
  delete this ;
}

inline int Log::isactive() {
  return *this->_inuse ;
}

#endif
